/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sample.tswebclient.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.faces.event.ActionEvent;
import javax.faces.event.AjaxBehaviorEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sample.tswebclient.cassandra.Application;
import sample.tswebclient.cassandra.CassandraManager;
import sun.misc.BASE64Encoder;


/**
 * Howto deploy a CDI bean in liferay with portlet faces:
 * http://www.liferay.com/community/wiki/-/wiki/Main/Developing+JSF+Portlets+with+CDI
 * 
 * @author ricky
 */
//@javax.faces.bean.ManagedBean(name = "applicationTreeBean")
//@javax.faces.bean.SessionScoped
@javax.inject.Named(value = "applicationTreeBean")
@javax.enterprise.context.SessionScoped
public class ApplicationTreeBean implements Serializable {

    private static final Logger logger = LoggerFactory.getLogger(ApplicationTreeBean.class);
    private String clickedFolder = null;
    private List<ApplicationItem> tree = null;
    private Map<String,Application> map = null;
    private Application selectedApplication = null;

    @javax.inject.Inject
    private CassandraManager manager;

    public ApplicationTreeBean() throws Exception {
        logger.info("creating...");
        tree = new ArrayList<>();
        map = new HashMap<>();
        clickedFolder = null;
    }
    
    @PostConstruct
    public void init() {
        createList(Application.root().getKey(), 0);
    }

    //static protected CassandraManager getManager() {
    //    return manager;
    //}

    public String getClickedFolder() {
        return clickedFolder;
    }

    public void setClickedFolder(String clickedFolder) {
        this.clickedFolder = clickedFolder;
    }

    public List<ApplicationItem> getTree() {
        return tree;
    }
    
    public Application getApplication(String key) {
        return this.map.get(key);
    }

    public boolean isOpenFolder(String key) {
        return clickedFolder != null && 
                (clickedFolder.startsWith(key + "/") || clickedFolder.equals(key));
    }

    public String encodeImage(byte[] img) {
        StringBuilder sb = new StringBuilder();
        sb.append("data:image/png;base64,");
        sb.append(new BASE64Encoder().encode(img));
        return sb.toString();
    }

    public Application getSelectedApplication() {
        return selectedApplication;
    }

    //
    // PRIVATE METHODS
    //

    private void createList(String key, int depth) {
        logger.info("createList {} {}", key, depth);
        // read the app an their children
        Application app = manager.readApplication(key,
                CassandraManager.ChildrenReadOption.FULL);
        logger.info("app: {}", app);
        // create the list
        for (Application child: app.getChildren()) {
            // add this child to the list
            tree.add(new ApplicationItem(child, depth));
            map.put(child.getKey(), child);
            // if it is selected => add its children too
            if (isOpenFolder(child.getKey())) {
                createList(child.getKey(), depth + 1);
            }
        }
    }

    //
    // ACTIONS
    //

    public void mouseOverListener(AjaxBehaviorEvent event) {
        String key = (String) event.getComponent().getAttributes().get("mouseKey");
        selectedApplication = map.get(key);
    }

    public void mouseOutListener(AjaxBehaviorEvent event) {
        selectedApplication = null;
    }

    public void listListener(ActionEvent event) {
        Object key = event.getComponent().getAttributes().get("clickedKey");
        if (key != null) {
            clickedFolder = key.toString();
        }
        logger.warn("ActionEvent doList " + clickedFolder);
        doList();
    }

    public String doList() {
        try {
            logger.debug("doList init...");
            tree.clear();
            map.clear();
            createList(Application.root().getKey(), 0);
        } catch (Exception e) {
            logger.error("createList error", e);
            FacesUtils.errorMessage(null, e);
            tree.clear();
            map.clear();
        }
        return "list";
    }

}
