/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sample.tswebclient.bean;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ResourceBundle;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;

/**
 *
 * @author ricky
 */
public class FacesUtils {
    
    final static public String GENERIC_ERROR_MESSAGE_ID = "internal_exception";

    /**
     * Private method that construct a message and add it to the faces
     * context. It uses ParametrizedFacesMessage for param subtitution.
     * @param clientId The clientId to assign the message
     * @param severity The severity of the message
     * @param summary The summary to put
     * @param summaryObjects The substitution to perform in summary
     * @param detail The detail to put
     * @param detailObjects The substitution to perform in detail
     */
    static private void constructParametrizedMessageReal(String clientId,
            FacesMessage.Severity severity,
            String summary, Object[] summaryObjects,
            String detail, Object[] detailObjects) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        ParametrizedFacesMessage mess = new ParametrizedFacesMessage();
        mess.setSeverity(severity);
        mess.setParametrizedDetail(detail, detailObjects);
        mess.setParametrizedSummary(summary, summaryObjects);
        facesContext.addMessage(clientId, mess);
    }
    
    /**
     * Method that returns true if the specified key exists in the
     * specified bundle. It's done cos ResourceBundle.containsKey(String)
     * doesn't exist before j2se6.
     * @param res The resource bundle to check the key in
     * @param key The String key to check if exists
     * @return true if exists, false if not
     */
    static private boolean containsKey(ResourceBundle res, String key) {
        try {
          return res.getObject(key) != null;
        }
        catch (Exception e) {
            return false;
        }
    }

    /**
     * Construct a message and add to the faces context. This method lookup
     * in default message bundle for possible localizations, if they
     * exists the localization is used.
     * @param clientId The clientId to assign the message
     * @param severity The severity of the message
     * @param summaryId Summary to put (or id for the localization)
     * @param summaryObjects Params for substitution
     * @param detailId Detail to put (or id for the localization)
     * @param detailObjects Params for substitution
     */
    static public void constructParametrizedMessage(String clientId,
            FacesMessage.Severity severity,
            String summaryId, Object[] summaryObjects,
            String detailId, Object[] detailObjects) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        String bundle = facesContext.getApplication().getMessageBundle();
        ResourceBundle resBundle = null;
        if (bundle != null) {
            resBundle = ResourceBundle.getBundle(bundle,
                    facesContext.getViewRoot().getLocale());
        }
        // get real strings
        String summary = summaryId;
        String detail = detailId;
        if (summaryId != null && resBundle != null && containsKey(resBundle, summaryId)) {
            summary = resBundle.getString(summaryId);
        }
        if (detailId != null && resBundle != null && containsKey(resBundle, detailId)) {
            detail = resBundle.getString(detailId);
        }
        // construct the messages
        constructParametrizedMessageReal(clientId, severity,
                summary, summaryObjects,
                detail, detailObjects);
    }

    /**
     * Construct a info message with no params.
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     */
    static public void infoMessage(String clientId, String summary) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_INFO,
                summary, null, null, null);
    }

    /**
     * Construct a info message with substitution params
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     * @param summaryObjects Params for substitution
     */
    static public void infoMessage(String clientId,
            String summary, Object[] summaryObjects) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_INFO,
                summary, summaryObjects, null, null);
    }
    
    /**
     * Construct a warning message without params.
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     */
    static public void warnMessage(String clientId, String summary) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_WARN,
                summary, null, null, null);
    }

    /**
     * Construct a warning message with params
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     * @param summaryObjects Params for substitution
     */
    static public void warnMessage(String clientId,
            String summary, Object[] summaryObjects) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_WARN,
                summary, summaryObjects, null, null);
    }
    
    /**
     * Construct a error message without params.
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     */
    static public void errorMessage(String clientId, String summary) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_ERROR,
                summary, null, null, null);
    }

    /**
     * Construct a error message with params.
     * @param clientId The client id to joing the message
     * @param summary The summary or localization id
     * @param summaryObjects Params for substitution
     */
    static public void errorMessage(String clientId,
            String summary, Object[] summaryObjects) {
        constructParametrizedMessage(clientId, FacesMessage.SEVERITY_ERROR,
                summary, summaryObjects, null, null);
    }
    
    /**
     * Construct a error message based in a exception. The idea is set
     * a generic (localized) message and then the e.getMessage().
     * @param clientId The client id to joing the message
     * @param exceptionId Message or id for internazionalization
     * @param e The exception
     */
    static public void errorMessage(String clientId,
            String exceptionId, Exception e) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        String bundle = facesContext.getApplication().getMessageBundle();
        ResourceBundle resBundle = null;
        if (bundle != null) {
            ResourceBundle.getBundle(bundle,
                    facesContext.getViewRoot().getLocale());
        }
        String mess = "";
        if (exceptionId != null && resBundle != null) {
            try {
                mess = resBundle.getString(exceptionId);
            }
            catch (Exception ex) {
                mess = exceptionId;
            }
        }
        mess += e.getLocalizedMessage();
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        constructParametrizedMessageReal(clientId, FacesMessage.SEVERITY_ERROR,
                mess, null, sw.toString(), null);
    }
    
    /**
     * Same as previous but using the default message id: job_calendar_exception
     * @param clientId The client id to joing the message
     * @param e The exception
     */
    static public void errorMessage(String clientId, Exception e) {
        errorMessage(clientId, GENERIC_ERROR_MESSAGE_ID, e);
    }

    /**
     * Return a parameter from the http request. It must be used in
     * commandLink components to get params. This method return the
     * parameter as is (as a String).
     * @param name The parameter name
     * @return The string value of the parameter or null
     */
    static public String getParameter(String name) {
        FacesContext context = FacesContext.getCurrentInstance();
        String id = null;
        if (context.getExternalContext().getRequest() instanceof HttpServletRequest) {
            HttpServletRequest req = (HttpServletRequest) context.getExternalContext().getRequest();
            id = req.getParameter(name);
        } else if (context.getExternalContext().getRequest() instanceof PortletRequest) {
            PortletRequest req = (PortletRequest) context.getExternalContext().getRequest();
            id = req.getParameter(name);
        }
        return id;
    }

}
