/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.tswebclient.cassandra;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author ricky
 */
public class Application implements Serializable {
    
    public enum Type {
        FOLDER,
        APPLICATION,
    };

    private String key = null;
    private String name = null;
    private Type type = null;
    private byte[] icon = null;
    private long writeIconTimestamp = 0L;
    private String description = null;
    private String path = null;
    private byte[] rdp = null;
    private long writeRdpTimestamp = 0L;
    private List<Application> children = null;

    protected Application(String key) {
        this.key = key;
        children = new ArrayList<>();
    }

    protected Application(String key, String name, Type type) {
        this.key = key;
        this.name = name;
        this.type = type;
        children = new ArrayList<>();
    }

    public Application(Application parent, String name, Type type) {
        if (parent.equals(root())) {
            this.key = new StringBuilder("/").append(name.toUpperCase()).toString();
        } else {
            this.key = new StringBuilder(parent.key).append("/").append(name.toUpperCase()).toString();
        }
        this.name = name;
        this.type = type;
        children = new ArrayList<>();
    }

    public Application(Application parent, String name, Type type,
            byte[] icon, String description, String path, byte[] rdp) {
        this(parent, name, type);
        this.description = description;
        this.icon = icon;
        this.path = path;
        this.rdp = rdp;
    }

    static public Application root() {
        return new Application("/", "ROOT", Type.FOLDER);
    }

    public String getKey() {
        return key;
    }
    
    public String getKeyEncoded() {
        try {
            return URLEncoder.encode(key, "UTF-8");
        } catch(UnsupportedEncodingException e) {
            return null;
        }
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public boolean isFolferType() {
        return Type.FOLDER.equals(type);
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    public byte[] getIcon() {
        return icon;
    }

    public void setIcon(byte[] icon) {
        this.icon = icon;
    }

    public byte[] getRdp() {
        return rdp;
    }

    public void setRdp(byte[] rdp) {
        this.rdp = rdp;
    }

    public long getWriteIconTimestamp() {
        return writeIconTimestamp;
    }

    public void setWriteIconTimestamp(long writeIconTimestamp) {
        this.writeIconTimestamp = writeIconTimestamp;
    }

    public long getWriteRdpTimestamp() {
        return writeRdpTimestamp;
    }

    public void setWriteRdpTimestamp(long writeRdpTimestamp) {
        this.writeRdpTimestamp = writeRdpTimestamp;
    }

    public List<Application> getChildren() {
        return children;
    }

    public String getParentKey() {
        return Application.getParentKey(key);
    }

    static public String getParentKey(String key) {
        int idx = key.lastIndexOf("/");
        if (key.equals("/")) {
            return null;
        } else if (idx == 0) {
            return "/";
        } else {
            return key.substring(0, idx);
        }
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof Application) {
            Application app = (Application) o;
            return this.key.equals(app.key);
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }

    private StringBuffer printPath(int depth, StringBuffer sb) {
        for (int i = 0; i < depth*2; i++) {
            sb.append(" ");
        }
        sb.append("|-> ");
        sb.append(key);
        sb.append(" (");
        sb.append(name);
        sb.append(" | ");
        sb.append(type);
        sb.append(")");
        sb.append(System.getProperty("line.separator"));
        for (Application child: children) {
            child.printPath(depth+1, sb);
        }
        return sb;
    }

    public String printPath() {
        StringBuffer sb = new StringBuffer();
        return printPath(0, sb).toString();
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("*** ");
        sb.append(this.getClass().toString());
        sb.append(System.getProperty("line.separator"));
        sb.append("key: ");
        sb.append(key);
        sb.append(System.getProperty("line.separator"));
        sb.append("name: ");
        sb.append(name);
        sb.append(System.getProperty("line.separator"));
        sb.append("type: ");
        sb.append(type);
        sb.append(System.getProperty("line.separator"));
        sb.append("description: ");
        sb.append(description);
        sb.append(System.getProperty("line.separator"));
        sb.append("icon: ");
        sb.append(icon);
        sb.append("-");
        sb.append(writeIconTimestamp);
        sb.append(System.getProperty("line.separator"));
        sb.append("rdp: ");
        sb.append(rdp);
        sb.append(System.getProperty("line.separator"));
        sb.append("path: ");
        sb.append(path);
        sb.append("-");
        sb.append(writeRdpTimestamp);
        for (Application child: children) {
            sb.append(System.getProperty("line.separator"));
            sb.append(" |-> ");
            sb.append(child.key);
            sb.append(" (");
            sb.append(child.name);
            sb.append(")");
        }
        sb.append(System.getProperty("line.separator"));
        return sb.toString();
    }
}
